﻿using System;
using System.Configuration;
using System.Linq;
using MongoDB.Driver;
using MongoDB.Driver.Builders;
using MongoDB.Driver.Linq;
using MongoSamples.Model;
using NUnit.Framework;

namespace MongoSamples
{
    [TestFixture]
    class CrudTests
    {
        private readonly MongoClient _client;

        public CrudTests()
        {
            _client = GetMongoClient();
        }

        [Test]
        public void Test_UpdateOneImage()
        {
            MongoDatabase db = GetDatabase("gallery");
            MongoCollection<Image> collection = db.GetCollection<Image>("images");

            var query = Query<Image>.EQ(img => img.Id, 10);

            Image existingImage = collection.FindOne(query);

            Assert.IsNotNull(existingImage);

            existingImage.Tags.Add("sample");

            collection.Save(existingImage);

            Image updatedImage = collection.FindOneAs<Image>(query);

            Assert.IsNotNull(updatedImage);

            Assert.IsTrue(updatedImage.Tags.Contains("sample"));
        }

        [Test]
        public void Test_UpdateMultipleImages()
        {
            MongoDatabase db = GetDatabase("gallery");
            MongoCollection<Image> collection = db.GetCollection<Image>("images");

            // Increase the height and width of all images with the tag "work"

            IMongoQuery query = Query<Image>.EQ(img => img.Tags, "work");
            UpdateBuilder<Image> update = Update<Image>.Mul(img => img.Height, 2).Mul(img => img.Width, 2);

            collection.Update(query, update, UpdateFlags.Multi);

            // Verify the update worked

            MongoCursor<Image> cursor = collection.Find(query);

            // By default, all documents in the "images" collection have a "height" of 480
            // and a "width" of 640.

            foreach (Image image in cursor)
            {
                Assert.IsTrue(image.Height > 480);
                Assert.IsTrue(image.Width > 640);
            }
        }

        [TearDown]
        public void TearDown()
        {
            MongoDatabase db = GetDatabase("gallery");
            MongoCollection<Image> collection = db.GetCollection<Image>("images");

            // Remove the "sample" tag from images

            IMongoQuery query1 = Query<Image>.Where(img => img.Tags.Contains("sample"));

            UpdateBuilder<Image> update1 = Update<Image>.Pull<string>(img => img.Tags, "sample");

            collection.Update(query1, update1, UpdateFlags.Multi);

            // By default, all documents in the "images" collection have a "height" of 480
            // and a "width" of 640. Reset the images to this state.

            IMongoQuery query2 = Query.Or(
                Query<Image>.GT(img => img.Width, 640),
                Query<Image>.GT(img => img.Height, 480)
                );

            UpdateBuilder<Image> update2 = Update<Image>.Set(img => img.Width, 640).Set(img => img.Height, 480);

            collection.Update(query2, update2, UpdateFlags.Multi);
        }

        private MongoDatabase GetDatabase(string dbName)
        {
            return _client.GetServer().GetDatabase(dbName);
        }

        private static MongoClient GetMongoClient()
        {
            // See http://docs.mongodb.org/manual/reference/connection-string/ for full details
            // on MongoDB connection string format and options.
            const string connectionStringTemplate = "mongodb://{0}:27017";

            string host = ConfigurationManager.AppSettings["Mongo.Host"];
            string connString = String.Format(connectionStringTemplate, host);

            MongoClient client = new MongoClient(connString);

            return client;
        }
    }
}
