﻿using System;
using System.Configuration;
using System.Linq;
using MongoDB.Driver;
using MongoDB.Driver.Builders;
using MongoSamples.Model;
using NUnit.Framework;

namespace MongoSamples
{
    /// <summary>
    /// Demonstrates how to access collections and write queries using the <c>Query</c> class
    /// provided by the MongoDB C# driver.
    /// </summary>
    [TestFixture]
    class QueryTests
    {
        private readonly MongoClient _client;

        public QueryTests()
        {
            _client = GetMongoClient();
        }

        [Test]
        public void Test_CanAccessAlbumCollection()
        {
            MongoDatabase db = GetDatabase("gallery");
            Assert.DoesNotThrow(() => db.GetCollection<Album>("albums"));
        }

        [Test]
        public void Test_Get10Albums()
        {
            MongoDatabase db = GetDatabase("gallery");
            MongoCollection<Album> collection = db.GetCollection<Album>("albums");

            // Shell: db.albums.find({_id:{$gt:0}});
            IMongoQuery query = Query<Album>.Where(al => al.Id > 0);

            MongoCursor<Album> cursor = collection.Find(query);
            cursor.Limit = 10;

            Assert.IsNotNull(cursor);

            int count = 0;

            foreach (Album album in cursor)
            {
                Assert.IsNotNull(album);
                Assert.Greater(album.Id, 0);
                count += 1;
            }

            Assert.IsTrue(count <= 10);
        }

        [Test]
        public void Test_CanAccessImagesCollection()
        {
            MongoDatabase db = GetDatabase("gallery");
            Assert.DoesNotThrow(() => db.GetCollection<Image>("images"));
        }

        [Test]
        public void Test_Get10Images()
        {
            MongoDatabase db = GetDatabase("gallery");
            MongoCollection<Image> collection = db.GetCollection<Image>("images");

            // Shell: db.images.find({_id:{$gt:0}});
            IMongoQuery query = Query.GT("_id", 0);

            MongoCursor<Image> cursor = collection.Find(query);
            cursor.Limit = 10;

            Assert.IsNotNull(cursor);

            int count = 0;

            foreach (Image image in cursor)
            {
                Assert.IsNotNull(image);
                Assert.Greater(image.Id, 0);
                count += 1;
            }

            Assert.IsTrue(count <= 10);
        }

        [Test]
        public void Test_GetCatImages()
        {
            MongoDatabase db = GetDatabase("gallery");
            MongoCollection<Image> collection = db.GetCollection<Image>("images");

            // Shell: db.images.find({tags:"cats"})
            IMongoQuery query = Query<Image>.Where(img => img.Tags.Contains("cats"));

            MongoCursor<Image> cursor = collection.Find(query);
            cursor.Limit = 10;

            Assert.IsNotNull(cursor);

            foreach (Image image in cursor)
            {
                Assert.IsNotNull(image);
                Assert.IsTrue(image.Tags.Contains("cats"));
            }
        }

        private MongoDatabase GetDatabase(string dbName)
        {
            return _client.GetServer().GetDatabase(dbName);
        }

        private static MongoClient GetMongoClient()
        {
            // See http://docs.mongodb.org/manual/reference/connection-string/ for full details
            // on MongoDB connection string format and options.
            const string connectionStringTemplate = "mongodb://{0}:27017";

            string host = ConfigurationManager.AppSettings["Mongo.Host"];
            string connString = String.Format(connectionStringTemplate, host);

            MongoClient client = new MongoClient(connString);

            return client;
        }
    }
}
